@@ -2,7 +2,7 @@ import click
 import os
 from thunder import utils
 
-OAUTH_URL = "https://console.thundercompute.com/?setup=cli"
+OAUTH_URL = "https://console.thundercompute.com/settings/tokens"
 CONSOLE_URL = "https://console.thundercompute.com/settings/billing"
 
 
@@ -80,7 +80,7 @@ def login():
         f.write(token)
     click.echo(
         click.style(
-            f"Success! Return to the console and add a payment method to claim your $20 of free credits: {CONSOLE_URL}",
+            f"Success! Return to the console and add a payment method: {CONSOLE_URL}",
             fg="green",
         )
     )
