@@ -44,14 +44,28 @@ try:
 except Exception:
     pass
 
+try:
+    VERSION = version("tnr")
+except Exception:
+    VERSION = "unknown"
+
 IS_WINDOWS = PLATFORM == "windows"
 
 if IS_WINDOWS:
     import win32security
     import ntsecuritycon as con
 
+IS_BINARY = getattr(sys, 'frozen', False) or hasattr(sys, '_MEIPASS')
+
+HEADERS = {
+    "Version": VERSION,
+    "Thunder-Client": "CLI-BINARY" if IS_BINARY else "CLI-PYTHON",
+    "Platform": PLATFORM,
+}
+
 
 session = requests.Session()
+session.headers.update(HEADERS)
 
 
 def setup_instance(token):
@@ -98,6 +112,188 @@ def get_next_id(token):
         return None, e
 
 
+def configure_thunder_virtualization(ssh, token, metadata, instance_id, latest_binary_hash=None):
+    """
+    Configure or remove Thunder Compute virtualization based on instance mode.
+    
+    For production mode instances, removes Thunder virtualization.
+    For prototyping mode instances, sets up Thunder virtualization with binary and config.
+    
+    Args:
+        ssh: SSH client connection
+        token: API token
+        metadata: Instance metadata containing GPU type and mode info
+        instance_id: Instance ID
+        latest_binary_hash: Latest binary hash for virtual GPU setup (optional)
+    
+    Returns:
+        bool: True if configuration was successful, False otherwise
+    """
+    try:
+        mode = metadata.get('mode', 'prototyping').lower()
+        is_production_mode = mode == 'production'
+        
+        if is_production_mode:
+            # For production instances, remove Thunder Compute virtualization
+            remote_config_path = '/home/ubuntu/.thunder/config.json'
+            remote_token_path = '/home/ubuntu/.thunder/token'
+            thunder_dir = '/etc/thunder'
+            
+            commands = [
+                # Clear ld.so.preload to remove Thunder library loading
+                'sudo rm -f /etc/ld.so.preload || true',
+                'sudo touch /etc/ld.so.preload',
+                'sudo chown root:root /etc/ld.so.preload',
+                'sudo chmod 644 /etc/ld.so.preload',
+                
+                # Remove Thunder binary and config files
+                'sudo rm -f /home/ubuntu/.thunder/libthunder.so || true',
+                f'sudo rm -f {remote_config_path} || true',
+                'sudo rm -rf /etc/thunder || true',
+                
+                # Keep the token file for API access
+                f'echo \'{token}\' > /tmp/token.tmp',
+                'sudo install -d -m 755 /home/ubuntu/.thunder',
+                f'sudo install -m 600 -o ubuntu -g ubuntu /tmp/token.tmp {remote_token_path}',
+                'rm -f /tmp/token.tmp'
+            ]
+            
+            command_string = ' && '.join(commands)
+            _, stdout, stderr = ssh.exec_command(command_string)
+            
+            exit_status = stdout.channel.recv_exit_status()
+            if exit_status != 0:
+                error_message = stderr.read().decode('utf-8')
+                return False, f"Failed to clean up Thunder virtualization: {error_message}"
+            
+            return True, "Successfully cleaned up Thunder virtualization for production mode"
+            
+        else:
+            # For prototyping instances, set up Thunder Compute virtualization
+            current_gpu_type = metadata.get('gpuType', 't4').lower()
+            # Check for existing config first
+            _, stdout, _ = ssh.exec_command(
+                "cat /home/ubuntu/.thunder/config.json 2>/dev/null || echo ''")
+            existing_config = stdout.read().decode().strip()
+            device_id = None
+            gpu_type = None
+            gpu_count = None
+            
+            try:
+                if existing_config:
+                    config_data = json.loads(existing_config)
+                    if config_data.get("deviceId") and int(config_data.get("deviceId")) > 0:
+                        device_id = config_data["deviceId"]
+                        gpu_type = config_data["gpuType"]
+                        gpu_count = config_data["gpuCount"]
+            except Exception:
+                # If we can't parse the existing config, we'll just get a new device ID
+                pass
+
+            update_binary = True
+            # Only get new device ID if we couldn't read it from existing config
+            if device_id is None:
+                device_id, error = get_next_id(token)
+                if error:
+                    return False, f"Could not grab next device ID: {error}"
+
+            if gpu_type is not None and gpu_count is not None:
+                if gpu_type == current_gpu_type and int(gpu_count) == int(metadata.get('numGpus', 1)):
+                    # Check if binary needs updating
+                    if latest_binary_hash:
+                        _, stdout, _ = ssh.exec_command(
+                            "md5sum /home/ubuntu/.thunder/libthunder.so 2>/dev/null || echo ''")
+                        existing_binary_hash = stdout.read().decode().strip().split(" ")[0]
+                        
+                        if existing_binary_hash == latest_binary_hash:
+                            update_binary = False
+                        else:
+                            update_binary = True
+                    else:
+                        # If no hash provided, assume update needed
+                        update_binary = True
+                else:
+                    # GPU config changed, force update
+                    update_binary = True
+
+            if update_binary:
+                config = {
+                    "instanceId": instance_id,
+                    "deviceId": device_id,
+                    "gpuType": current_gpu_type,
+                    "gpuCount": int(metadata.get('numGpus', 1)),
+                }
+
+                # Update binary and write config in a single command
+                remote_path = '/home/ubuntu/.thunder/libthunder.so'
+                remote_config_path = '/home/ubuntu/.thunder/config.json'
+                remote_token_path = '/home/ubuntu/.thunder/token'
+                thunder_dir = '/etc/thunder'
+                thunder_lib_path = f'{thunder_dir}/libthunder.so'
+                config_json = json.dumps(config, indent=4)
+
+                commands = [
+                    # Create directories with explicit permissions
+                    f'sudo install -d -m 755 {thunder_dir}',
+                    'sudo install -d -m 755 /home/ubuntu/.thunder',
+
+                    # Back up existing preload file
+                    'sudo cp -a /etc/ld.so.preload /etc/ld.so.preload.bak || true',
+
+                    # Download to a temporary file first
+                    f'curl -L https://storage.googleapis.com/client-binary/client_linux_x86_64 -o /tmp/libthunder.tmp',
+                    f'sudo install -m 755 -o root -g root /tmp/libthunder.tmp {remote_path}',
+                    'rm -f /tmp/libthunder.tmp',
+                    f'[ -f "{remote_path}" ] && [ -s "{remote_path}" ] || (echo "Download failed" && exit 1)',
+
+                    # Create symlink for libthunder.so
+                    f'sudo rm -f {thunder_lib_path} || true',
+                    f'sudo ln -sf {remote_path} {thunder_lib_path}',
+                    'sudo chown root:root /etc/thunder/libthunder.so',
+                    'sudo chmod 755 /etc/thunder/libthunder.so',
+
+                    # Update preload file to use the symlinked path
+                    f'echo "{thunder_lib_path}" | sudo tee /etc/ld.so.preload.new',
+                    'sudo chown root:root /etc/ld.so.preload.new',
+                    'sudo chmod 644 /etc/ld.so.preload.new',
+                    'sudo mv /etc/ld.so.preload.new /etc/ld.so.preload',
+
+                    # Write config files
+                    f'echo \'{config_json}\' > /tmp/config.tmp',
+                    f'sudo install -m 644 -o root -g root /tmp/config.tmp {remote_config_path}',
+                    'rm -f /tmp/config.tmp',
+
+                    f'echo \'{token}\' > /tmp/token.tmp',
+                    f'sudo install -m 600 -o root -g root /tmp/token.tmp {remote_token_path}',
+                    'rm -f /tmp/token.tmp',
+
+                    # Symlink config and token to .thunder
+                    'sudo rm -f /etc/thunder/config.json || true',
+                    f'sudo ln -sf {remote_config_path} /etc/thunder/config.json',
+                    'sudo chown root:root /etc/thunder/config.json',
+                    'sudo chmod 644 /etc/thunder/config.json',
+                    'sudo rm -f /etc/thunder/token || true',
+                    f'sudo ln -sf {remote_token_path} /etc/thunder/token',
+                    'sudo chown root:root /etc/thunder/token',
+                    'sudo chmod 644 /etc/thunder/token'
+                ]
+
+                command_string = ' && '.join(commands)
+                _, stdout, stderr = ssh.exec_command(command_string)
+
+                exit_status = stdout.channel.recv_exit_status()
+                if exit_status != 0:
+                    error_message = stderr.read().decode('utf-8')
+                    return False, f"Failed to update binary and write config: {error_message}"
+                
+                return True, "Successfully configured Thunder virtualization"
+            else:
+                return True, "Thunder virtualization already up to date"
+                
+    except Exception as e:
+        return False, f"Failed to configure Thunder virtualization: {str(e)}"
+
+
 def remove_host_key(device_ip):
     try:
         subprocess.run(
@@ -201,19 +397,21 @@ def get_instances(token, use_cache=True, update_ips=False):
 get_instances.cache = None
 
 
-def create_instance(token, cpu_cores, gpu_type, template, num_gpus, disk_size_gb):
+def create_instance(token, cpu_cores, gpu_type, template_id, num_gpus, disk_size_gb, mode='prototyping'):
     endpoint = f"{BASEURL}/instances/create"
     payload = {
         "cpu_cores": cpu_cores,
         "gpu_type": gpu_type,
-        "template": template,
+        "template": template_id,
         "num_gpus": num_gpus,
         "disk_size_gb": disk_size_gb,
+        "mode": mode,
     }
+    
     try:
         response = session.post(
             endpoint,
-            headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM},
+            headers={"Authorization": f"Bearer {token}"},
             json=payload,
             timeout=30
         )
@@ -233,7 +431,7 @@ def delete_instance(instance_id, token):
     endpoint = f"{BASEURL}/instances/{instance_id}/delete"
     try:
         response = session.post(
-            endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM}, timeout=30
+            endpoint, headers={"Authorization": f"Bearer {token}"}, timeout=30
         )
         if response.status_code != 200:
             return False, response.text
@@ -247,7 +445,7 @@ def start_instance(instance_id, token):
     endpoint = f"{BASEURL}/instances/{instance_id}/up"
     try:
         response = session.post(
-            endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM}, timeout=30
+            endpoint, headers={"Authorization": f"Bearer {token}"}, timeout=90
         )
         if response.status_code != 200:
             return False, response.text
@@ -261,7 +459,7 @@ def stop_instance(instance_id, token):
     endpoint = f"{BASEURL}/instances/{instance_id}/down"
     try:
         response = session.post(
-            endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM}, timeout=30
+            endpoint, headers={"Authorization": f"Bearer {token}"}, timeout=30
         )
         if response.status_code != 200:
             return False, response.text
@@ -275,7 +473,7 @@ def get_active_sessions(token):
     endpoint = f"{BASEURL}/active_sessions"
     try:
         response = session.get(
-            endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM}, timeout=30
+            endpoint, headers={"Authorization": f"Bearer {token}"}, timeout=30
         )
         if response.status_code != 200:
             return None, []
@@ -292,7 +490,7 @@ def add_key_to_instance(instance_id, token):
     endpoint = f"{BASEURL}/instances/{instance_id}/add_key"
     try:
         response = session.post(
-            endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM}, timeout=30
+            endpoint, headers={"Authorization": f"Bearer {token}"}, timeout=30
         )
         if response.status_code != 200:
             return False, f"Failed to add key to instance {instance_id}: {response.text}"
@@ -310,9 +508,7 @@ def get_ip(token):
     endpoint = f"{BASEURL}/current_ip"
     try:
         response = session.get(
-            endpoint,
-            headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM},
-            timeout=30,
+            endpoint, headers={"Authorization": f"Bearer {token}"}, timeout=30
         )
         if response.status_code != 200:
             return False, response.text
@@ -499,7 +695,7 @@ def add_instance_to_ssh_config(hostname, key_path, host_alias=None, port=None):
         f"Host {host_alias}\n",
         f"    HostName {hostname}\n",
         f"    User ubuntu\n",
-        f"    IdentityFile {key_path}\n",
+        f"    IdentityFile \"{key_path}\"\n",
         f"    IdentitiesOnly yes\n",
         f"    StrictHostKeyChecking no\n",
     ]
@@ -577,7 +773,7 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
                 if not has_strict_checking:
                     new_lines.append("    StrictHostKeyChecking no\n")
                 if keyfile and not updated_key:
-                    new_lines.append(f"    IdentityFile {keyfile}\n")
+                    new_lines.append(f"    IdentityFile \"{keyfile}\"\n")
             in_target_host = (stripped == f"Host {instance_name}")
             has_strict_checking = False
             updated_key = False if keyfile else True
@@ -592,7 +788,7 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
             if stripped.startswith("StrictHostKeyChecking"):
                 has_strict_checking = True
             if keyfile and stripped.startswith("IdentityFile"):
-                new_lines.append(f"    IdentityFile {keyfile}\n")
+                new_lines.append(f"    IdentityFile \"{keyfile}\"\n")
                 updated_key = True
                 continue
 
@@ -603,16 +799,16 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
         if not has_strict_checking:
             new_lines.append("    StrictHostKeyChecking no\n")
         if keyfile and not updated_key:
-            new_lines.append(f"    IdentityFile {keyfile}\n")
+            new_lines.append(f"    IdentityFile \"{keyfile}\"\n")
 
     if updated_ip:
         write_ssh_config(new_lines)
 
 
 def validate_token(token):
-    endpoint = f"https://api.thundercompute.com:8443/uid"
+    endpoint = f"{BASEURL}/uid"
     response = session.get(
-        endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM})
+        endpoint, headers={"Authorization": f"Bearer {token}"})
 
     if response.status_code == 200:
         return True, None
@@ -634,7 +830,6 @@ async def get_latest_client_binary_hash_async():
                 latest_binary_hash = latest_metadata.get("metadata", {}).get("hash")
     except Exception as e:
         # Log or handle the error appropriately if needed, for now just return None
-        print(f"Error fetching latest client binary hash: {e}") # Optional: Log the error
         pass # Keep latest_binary_hash as None
 
     return latest_binary_hash
@@ -698,8 +893,8 @@ def get_instance_id(token):
 
 def get_uid(token):
     endpoint = f"{BASEURL}/uid"
-    response = requests.get(
-        endpoint, headers={"Authorization": f"Bearer {token}", "Platform": PLATFORM})
+    response = session.get(
+        endpoint, headers={"Authorization": f"Bearer {token}"})
 
     if response.status_code != 200:
         raise click.ClickException(
@@ -736,70 +931,39 @@ def modify_instance(instance_id: str, payload: dict, token: str) -> Tuple[bool,
 # === Template Information ===
 
 
-TEMPLATE_INFO_CACHE_TTL = 3600  # 1 hour in seconds
-
-
-def get_template_info_cache_file():
-    basedir = join(os.path.expanduser("~"), ".thunder", "cache")
-    if not os.path.isdir(basedir):
-        os.makedirs(basedir)
-    return join(basedir, "template_info.json")
-
-
-def read_template_info_cache():
-    cache_file = get_template_info_cache_file()
-    try:
-        if os.path.exists(cache_file):
-            with open(cache_file) as f:
-                cached = json.load(f)
-                if cached.get('timestamp', 0) > time.time() - TEMPLATE_INFO_CACHE_TTL:
-                    return cached.get('templates', {})
-                else:
-                    return None
-    except Exception as e:
-        return None
-
-
-def delete_template_info_cache():
-    cache_file = get_template_info_cache_file()
-    if os.path.exists(cache_file):
-        os.remove(cache_file)
-
-
 def get_template_info(token=None):
     """Fetch template information from the API"""
     try:
         # Use production endpoint, fallback to localhost for development
         api_endpoint = f"{BASEURL}/thunder-templates"
-        cache = read_template_info_cache()
-        if cache:
-            templates = cache
+        templates = {}
+        headers = {"Authorization": f"Bearer {token}"} if token else {}
+        response = requests.get(api_endpoint, headers=headers, timeout=5)
+        if response.status_code == 200:
+            templates = response.json()
         else:
-            headers = {"Authorization": f"Bearer {token}"} if token else {}
-            response = requests.get(api_endpoint, headers=headers, timeout=5)
-            if response.status_code == 200:
-                templates = response.json()
-
-            with open(get_template_info_cache_file(), 'w') as f:
-                timestamp = time.time()
-                json.dump({
-                    'timestamp': timestamp,
-                    'templates': templates
-                }, f)
+            templates = {}
 
         # Extract template information
         template_info = []
 
         for template_name, template_data in templates.items():
-            template_info.append({
+            info = {
                 "name": template_name,
                 "default": template_data.get("default", True),
-                "displayName": template_data['displayName'],
+                "displayName": template_data.get('displayName', template_name),
+                "extendedDescription": template_data.get('extendedDescription', ''),
                 "defaultStorage": template_data.get('defaultSpecs', {}).get('storage', 100),
                 "defaultSpecs": template_data.get('defaultSpecs', {}),
-                "openPorts": template_data['openPorts'],
-                "automountFolders": template_data['automountFolders'][0] if template_data['automountFolders'] else None
-            })
+                "openPorts": template_data.get('openPorts', []),
+                "automountFolders": template_data['automountFolders'][0] if template_data.get('automountFolders') else None,
+                "imageId": template_name
+            }
+            if template_data.get("imageId"):
+                info["isSnapshot"] = True
+                info["name"] = template_data.get("displayName")
+                info["imageId"] = template_data.get("imageId")
+            template_info.append(info)
 
         return template_info
     except Exception as e:
@@ -1119,14 +1283,13 @@ def check_meets_min_version(force_check=False):
 
 def check_cli_up_to_date(force_check=False):
     """Checks if the running CLI meets minimum version and is the latest hash/version."""
-    is_binary = getattr(sys, 'frozen', False) or hasattr(sys, '_MEIPASS')
 
     meets_min, min_details = check_meets_min_version(force_check=force_check)
 
     is_latest = True  # Default to true
     latest_details = None
 
-    if is_binary:
+    if IS_BINARY:
         is_latest, latest_details = check_binary_hash(force_check=force_check)
     else:
         # For pip installs, we currently only enforce minimum.
